/**
 * Setup CO2 Sensor
 * Configure sampling rate here
 *
 * @param co2Sensor  CO2 Sensor
 * @return Status of Co2 Sensor initialisation
 */
 bool setupCO2(CCS811 co2Sensor) {
  if (co2Sensor.begin() != 0) {
    Serial.println("[-] CO2 Init Failed");
    return false;
  }

  /**
   * @brief Set measurement cycle
   * @param cycle:in typedef enum{
   *     eClosed,      // Idle (Measurements are disabled in this mode)
   *     eCycle_1s,    // Constant power mode, IAQ measurement every second
   *     eCycle_10s,   // Pulse heating mode IAQ measurement every 10 seconds
   *     eCycle_60s,   // Low power pulse heating mode IAQ measurement every 60 seconds
   *     eCycle_250ms  // Constant power mode, sensor measurement every 250ms
   *   }eCycle_t;
   */
  co2Sensor.setMeasCycle(co2Sensor.eCycle_250ms);
  return true;
}

/**
 * Get CO2 reading
 *
 * @param co2Sensor CO2 Sensor
 * @returns CO2 level (ppm) or -1 on error 
 */
double getCo2(CCS811 co2Sensor) {
  double c = -1;
  
  if(co2Sensor.checkDataReady() == true){
    c = co2Sensor.getCO2PPM();
  }

  co2Sensor.writeBaseLine(0x847B);
  return c; // -1 on error
}