/**
 * Get data from PMS Partical Sensor
 *
 * @param Stream s PMS Serial Connection
 * @param pms5003data data struct data PMS sensor Data
 * @modifies struct data PMS Sensor Data
 * @returns boolean Data Read Success status
 */
boolean readPMSdata(Stream *s, struct pms5003data *data) {
  if (!s->available()) {
    return false;
  }
  
  // Read a byte at a time until we get to the special '0x42' start-byte
  if (s->peek() != 0x42) {
    s->read();
    return false;
  }
 
  // Now read all 32 bytes
  if (s->available() < 32) {
    return false;
  }
    
  uint8_t buffer[32];    
  uint32_t sum = 0;
  s->readBytes(buffer, 32);
 
  // get checksum ready
  for (uint8_t i=0; i<30; i++) {
    sum += buffer[i];
  }
  
  // The data comes in endian'd, this solves it so it works on all platforms
  uint16_t buffer_u16[15];
  for (uint8_t i=0; i<15; i++) {
    buffer_u16[i] = buffer[2 + i*2 + 1];
    buffer_u16[i] += (buffer[2 + i*2] << 8);
  }

  // Struct it
  memcpy((void *)data, (void *)buffer_u16, 30);
 
  if (sum != data->checksum) { // Checksum failed!
    if (DEBUG) {
        Serial.print("\n Checksum failure... ");
        Serial.print("Expected: "); Serial.print(sum);
        Serial.print(", Got: "); Serial.print(data->checksum);
    }
    return false;
  }

  return true;
}
