/**
 * Setup Humidity / Temperature Sensor
 * Accuracy of SHT can be set here
 * 
 * @return sht on succesful initialisation
 */
 bool setupSHT(SHTSensor* sht) {
  Wire.begin();

  if (!sht->init()) {
    Serial.println("[-] SHT Init Failed");
    return false;
  }
  
  sht->setAccuracy(SHTSensor::SHT_ACCURACY_MEDIUM);
  return true;
}

/**
 * Get temperature from SHT
 *
 * @param sht SHT Sensor
 * @return temperature, -300 on read error
 */
double getTemperature(SHTSensor* sht) {
  return (sht->readSample() ? sht->getTemperature() : -300);
}

/**
 * Get humidity from SHT
 *
 * @param sht SHT Sensor
 * @return humidity, -1 on read error
 */
double getHumidity(SHTSensor* sht) {
  return (sht->readSample() ? sht->getHumidity() : -1);
}
