# Raspberry Pi Gateway / Reciever

## System Requirements
**Hardware**  
System is designed to run on `Raspberry Pi 3 or later`, with a `Ubuntu 64-bit` OS, however other OS' and similar hardware are likely to work.

This system makes use of the `443MHz` radio frequency for LORA transmission. Depending on your geographical location, you may require a different tranciever that operates at a permitted frequency.

Full B.O.M. and wiring instructions on parent [README](../README.md) 

**Software**
OS & Software Packages
- Ubuntu 64-bit (Pi Edition) [Install Guide](https://ubuntu.com/download/raspberry-pi)
- Python 3 [Install Guide](https://projects.raspberrypi.org/en/projects/generic-python-install-python3#:~:text=Raspberry%20Pi%20OS%20and%20other%20Linux%20(Debian%2Dbased)%20distributions)
- Python 3 Pip (e.g. `sudo apt install python3-pip -y`)
- Rabbit MQ Server [Install Guide](https://www.rabbitmq.com/install-debian.html#apt-quick-start-cloudsmith)

Python Packages, install with pip [Install Guide](https://packaging.python.org/en/latest/tutorials/installing-packages/#installing-from-pypi)
- requests
- pika 

Python Packages, included in repo
- SX127x (Lora Package)

## System Overview
Gateway sits within a deployment area, it can be run offline, or with an internet connection. The gateway can run the [API](https://gitlab.cis.strath.ac.uk/cs408-environmental-monitoring-independent-of-existing-infrastructure/public-api) locally or it can be on a separate host. For an offline deployment you may choose to connect the Pi to a monitor, keyboard and mouse in order to interact with the frontend, for this you'll require the a graphical desktop environment, e.g. Ubuntu Desktop. 

You may also choose to build a local area network, such that other computers can connect to the frontend and query the api in entirely-offline deployments. 

**What does this component do?**  
Gateway node listens for and consumes messages from LORA. 

Sensor data is consumed and placed on a queue, in preparation for transmission to the API.

The Gateway will response with appropriate ack/nack messages to gateway nodes.

Rabbit MQ is the chosen message broker server.

## Configuration
First-Time installation & Test
1. Clone Repo
2. Create Directory `/usr/share/emiei`
3. Copy contents of `./RaspberryPi_Receiver` to `/usr/share/emiei/`
4. Configure `secrets.py` - Provide Gateway API Key, Rabbit MQ authentication, HTTP Basic Auth If Required
5. Make additonal configuration changes in `Pi_Receiver.py` if required.
6. Test program with `/usr/bin/python3 /usr/share/emiei/Pi_Receiver.py -f 433 -b BW125 -s 7`
7. Inspect log file at `Pi_Receiver.log` in case of issues
8. Use `CTRL+C` to quit

## Service Install
To register as service, a sample systemd config file has been provided
1. Copy `emiei-gateway.service` to `/lib/systemd/system/`
2. Configure `<Target User>` and `<Target Group>`
3. Register service by running `sudo systemctl daemon-reload`
4. Test service with `sudo service emiei-gateway start`
5. Enable service at startup with `sudo systemctl enable emiei-gateway`